@@ -4,8 +4,23 @@ import logging
 import struct
 import sys
 from collections import namedtuple
+try:
+    from collections import OrderedDict
+except ImportError:
+    try:
+        # ordereddict available on pypi for Python < 2.7
+        from ordereddict import OrderedDict
+    except ImportError:
+        # Otherwise fall back on normal dict
+        OrderedDict = dict
 from copy import copy
 import numpy as np
+from datetime import datetime, timedelta
+import tempfile
+try:
+    import pytz
+except ImportError:
+    pytz = None
 
 
 log = logging.getLogger(__name__)
@@ -60,6 +75,13 @@ tdsDataTypes.update({
 })
 
 
+if pytz:
+    # Use UTC time zone if pytz is installed
+    timezone = pytz.utc
+else:
+    timezone = None
+
+
 def read_string(file):
     """Read a string from a tdms file
 
@@ -75,11 +97,17 @@ def read_type(file, data_type, endianness):
     """Read a value from the file of the specified data type"""
 
     if data_type.name == 'tdsTypeTimeStamp':
-        # Time stamps are stored as number of seconds since the epoch
-        # and number of 2^-64 fractions of a second
+        # Time stamps are stored as number of seconds since
+        # 01/01/1904 00:00:00.00 UTC, ignoring leap seconds,
+        # and number of 2^-64 fractions of a second.
+        # Note that the TDMS epoch is not the Unix epoch.
         s = file.read(data_type.length)
         (s_frac, s) = struct.unpack('%s%s' % (endianness, data_type.struct), s)
-        return (s, s_frac * (2 ** -64))
+        tdms_start = datetime(1904, 1, 1, 0, 0, 0, tzinfo=timezone)
+        ms = float(s_frac) * 5 ** 6 / 2 ** 58
+        # Adding timedelta with seconds ignores leap
+        # seconds, so this is correct
+        return tdms_start + timedelta(seconds=s) + timedelta(microseconds=ms)
     elif None not in (data_type.struct, data_type.length):
         s = file.read(data_type.length)
         return struct.unpack('%s%s' % (endianness, data_type.struct), s)[0]
@@ -95,16 +123,21 @@ class TdmsFile(object):
 
     """
 
-    def __init__(self, file):
+    def __init__(self, file, memmap_dir=None):
         """Initialise a new TDMS file object, reading all data.
 
         :param file: Either the path to the tdms file to read or an already
             opened file.
-
+        :param memmap_dir: The directory to store memmapped data files in,
+            or None to read data into memory. The data files are created
+            as temporary files and are deleted when the channel data is no
+            longer used. tempfile.gettempdir() can be used to get the default
+            temporary file directory.
         """
 
         self.segments = []
-        self.objects = {}
+        self.objects = OrderedDict()
+        self.memmap_dir = memmap_dir
 
         if hasattr(file, "read"):
             # Is a file
@@ -135,7 +168,7 @@ class TdmsFile(object):
 
         # Allocate space for data
         for object in self.objects.values():
-            object._initialise_data()
+            object._initialise_data(memmap_dir=self.memmap_dir)
 
         # Now actually read all the data
         for segment in self.segments:
@@ -194,7 +227,10 @@ class TdmsFile(object):
         """
 
         path = self._path(group)
-        return [self.objects[p] for p in self.objects if p.startswith(path)]
+        return [
+            self.objects[p]
+            for p in self.objects
+            if p.startswith(path + '/')]
 
     def channel_data(self, group, channel):
         """Get the data for a channel
@@ -206,7 +242,7 @@ class TdmsFile(object):
 
         """
 
-        return self.objects[self._path(group, channel)].data
+        return self.object(group, channel).data
 
 
 class _TdmsSegment(object):
@@ -234,7 +270,7 @@ class _TdmsSegment(object):
         s = f.read(4)
         toc_mask = struct.unpack('<i', s)[0]
 
-        self.toc = {}
+        self.toc = OrderedDict()
         for property in tocProperties.keys():
             self.toc[property] = (toc_mask & tocProperties[property]) != 0
             log.debug("Property %s is %s" % (property, self.toc[property]))
@@ -466,7 +502,7 @@ class TdmsObject(object):
     def __init__(self, path):
         self.path = path
         self.data = None
-        self.properties = {}
+        self.properties = OrderedDict()
         self.data_type = None
         self.dimension = 1
         self.number_values = 0
@@ -485,7 +521,11 @@ class TdmsObject(object):
 
         """
 
-        return self.properties[property_name]
+        try:
+            return self.properties[property_name]
+        except KeyError:
+            raise KeyError(
+                "Object does not have property '%s'" % property_name)
 
     def time_track(self):
         """Return an array of time for this channel
@@ -509,16 +549,26 @@ class TdmsObject(object):
                 offset + (len(self.data) - 1) * increment,
                 len(self.data))
 
-    def _initialise_data(self):
+    def _initialise_data(self, memmap_dir=None):
         """Initialise data array to zeros"""
 
-        if self.number_values > 0:
-            if self.data_type.nptype is None:
-                self.data = []
+        if self.number_values == 0:
+            pass
+        elif self.data_type.nptype is None:
+            self.data = []
+        else:
+            if memmap_dir:
+                memmap_file = tempfile.NamedTemporaryFile(
+                        mode='w+b', prefix="nptdms_", dir=memmap_dir)
+                self.data = np.memmap(
+                        memmap_file.name,
+                        mode='w+',
+                        shape=(self.number_values,),
+                        dtype=self.data_type.nptype)
             else:
                 self.data = np.zeros(
                         self.number_values, dtype=self.data_type.nptype)
-                self._data_insert_position = 0
+            self._data_insert_position = 0
 
     def _update_data(self, new_data):
         """Update the object data with a new array of data"""
@@ -552,7 +602,7 @@ class _TdmsSegmentObject(object):
         self.has_data = True
         self.data_type = None
         self.dimension = 1
-        self.properties = {}
+        self.properties = OrderedDict()
 
     def _read_metadata(self, f):
         """Read object metadata and update object information"""
