@@ -6,6 +6,9 @@ import logging
 import binascii
 import struct
 import tempfile
+from datetime import datetime
+import tempfile
+
 from nptdms import tdms
 
 
@@ -64,10 +67,10 @@ class TestFile(object):
         return binascii.unhexlify(hex_data.replace(" ", "").
                 replace("\n", "").encode('utf-8'))
 
-    def load(self):
+    def load(self, *args, **kwargs):
         self.file.write(self.data)
         self.file.seek(0)
-        return tdms.TdmsFile(self.file)
+        return tdms.TdmsFile(self.file, *args, **kwargs)
 
 
 class TDMSTestClass(unittest.TestCase):
@@ -168,13 +171,13 @@ class TDMSTestClass(unittest.TestCase):
 
         test_file = TestFile()
         test_file.add_segment(*self.basic_segment())
-        tdms = test_file.load()
+        tdmsData = test_file.load()
 
-        data = tdms.channel_data("Group", "Channel1")
+        data = tdmsData.channel_data("Group", "Channel1")
         self.assertEqual(len(data), 2)
         self.assertEqual(data[0], 1)
         self.assertEqual(data[1], 2)
-        data = tdms.channel_data("Group", "Channel2")
+        data = tdmsData.channel_data("Group", "Channel2")
         self.assertEqual(len(data), 2)
         self.assertEqual(data[0], 3)
         self.assertEqual(data[1], 4)
@@ -184,9 +187,9 @@ class TDMSTestClass(unittest.TestCase):
 
         test_file = TestFile()
         test_file.add_segment(*self.basic_segment())
-        tdms = test_file.load()
+        tdmsData = test_file.load()
 
-        object = tdms.object("Group")
+        object = tdmsData.object("Group")
         self.assertEqual(object.property("num"), 10)
 
     def test_no_metadata_segment(self):
@@ -204,12 +207,12 @@ class TDMSTestClass(unittest.TestCase):
         )
         toc = ("kTocRawData")
         test_file.add_segment('', data, toc)
-        tdms = test_file.load()
+        tdmsData = test_file.load()
 
-        data = tdms.channel_data("Group", "Channel1")
+        data = tdmsData.channel_data("Group", "Channel1")
         self.assertEqual(len(data), 4)
         self.assertTrue(all(data == [1, 2, 5, 6]))
-        data = tdms.channel_data("Group", "Channel2")
+        data = tdmsData.channel_data("Group", "Channel2")
         self.assertEqual(len(data), 4)
         self.assertTrue(all(data == [3, 4, 7, 8]))
 
@@ -262,12 +265,12 @@ class TDMSTestClass(unittest.TestCase):
             "00 00 00 00"
             "00 00 00 00")
         test_file.add_segment(metadata, data, toc)
-        tdms = test_file.load()
+        tdmsData = test_file.load()
 
-        data = tdms.channel_data("Group", "Channel1")
+        data = tdmsData.channel_data("Group", "Channel1")
         self.assertEqual(len(data), 4)
         self.assertTrue(all(data == [1, 2, 5, 6]))
-        data = tdms.channel_data("Group", "Channel2")
+        data = tdmsData.channel_data("Group", "Channel2")
         self.assertEqual(len(data), 4)
         self.assertTrue(all(data == [3, 4, 7, 8]))
 
@@ -307,15 +310,15 @@ class TDMSTestClass(unittest.TestCase):
             "0A 00 00 00"
         )
         test_file.add_segment(metadata, data, toc)
-        tdms = test_file.load()
+        tdmsData = test_file.load()
 
-        data = tdms.channel_data("Group", "Channel1")
+        data = tdmsData.channel_data("Group", "Channel1")
         self.assertEqual(len(data), 4)
         self.assertTrue(all(data == [1, 2, 5, 6]))
-        data = tdms.channel_data("Group", "Channel2")
+        data = tdmsData.channel_data("Group", "Channel2")
         self.assertEqual(len(data), 4)
         self.assertTrue(all(data == [3, 4, 7, 8]))
-        data = tdms.channel_data("Group", "Voltage")
+        data = tdmsData.channel_data("Group", "Voltage")
         self.assertEqual(len(data), 2)
         self.assertTrue(all(data == [9, 10]))
 
@@ -354,11 +357,11 @@ class TDMSTestClass(unittest.TestCase):
             "0A 00 00 00"
         )
         test_file.add_segment(metadata, data, toc)
-        tdms = test_file.load()
-        data = tdms.channel_data("Group", "Channel1")
+        tdmsData = test_file.load()
+        data = tdmsData.channel_data("Group", "Channel1")
         self.assertEqual(len(data), 4)
         self.assertTrue(all(data == [1, 2, 5, 6]))
-        data = tdms.channel_data("Group", "Channel2")
+        data = tdmsData.channel_data("Group", "Channel2")
         self.assertEqual(len(data), 6)
         self.assertTrue(all(data == [3, 4, 7, 8, 9, 10]))
 
@@ -393,11 +396,11 @@ class TDMSTestClass(unittest.TestCase):
             "06 00 00 00"
         )
         test_file.add_segment(metadata, data, toc)
-        tdms = test_file.load()
-        data = tdms.channel_data("Group", "Channel1")
+        tdmsData = test_file.load()
+        data = tdmsData.channel_data("Group", "Channel1")
         self.assertEqual(len(data), 4)
         self.assertTrue(all(data == [1, 2, 5, 6]))
-        data = tdms.channel_data("Group", "Channel2")
+        data = tdmsData.channel_data("Group", "Channel2")
         self.assertEqual(len(data), 2)
         self.assertTrue(all(data == [3, 4]))
 
@@ -414,12 +417,12 @@ class TDMSTestClass(unittest.TestCase):
             "08 00 00 00"
         )
         test_file.add_segment(metadata, data, toc)
-        tdms = test_file.load()
+        tdmsData = test_file.load()
 
-        data = tdms.channel_data("Group", "Channel1")
+        data = tdmsData.channel_data("Group", "Channel1")
         self.assertEqual(len(data), 4)
         self.assertTrue(all(data == [1, 2, 5, 6]))
-        data = tdms.channel_data("Group", "Channel2")
+        data = tdmsData.channel_data("Group", "Channel2")
         self.assertEqual(len(data), 4)
         self.assertTrue(all(data == [3, 4, 7, 8]))
 
@@ -430,13 +433,13 @@ class TDMSTestClass(unittest.TestCase):
         (metadata, data, toc) = self.basic_segment()
         toc = toc + ("kTocInterleavedData", )
         test_file.add_segment(metadata, data, toc)
-        tdms = test_file.load()
+        tdmsData = test_file.load()
 
-        data = tdms.channel_data("Group", "Channel1")
+        data = tdmsData.channel_data("Group", "Channel1")
         self.assertEqual(len(data), 2)
         self.assertEqual(data[0], 1)
         self.assertEqual(data[1], 3)
-        data = tdms.channel_data("Group", "Channel2")
+        data = tdmsData.channel_data("Group", "Channel2")
         self.assertEqual(len(data), 2)
         self.assertEqual(data[0], 2)
         self.assertEqual(data[1], 4)
@@ -445,6 +448,26 @@ class TDMSTestClass(unittest.TestCase):
         """Test reading contiguous and interleaved timestamp data,
         which isn't read by numpy"""
 
+
+        times = [
+            datetime(2012, 8, 23, 0, 0, 0, 123, tzinfo=tdms.timezone),
+            datetime(2012, 8, 23, 1, 2, 3, 456, tzinfo=tdms.timezone),
+            datetime(2012, 8, 23, 12, 0, 0, 0, tzinfo=tdms.timezone),
+            datetime(2012, 8, 23, 12, 2, 3, 9999, tzinfo=tdms.timezone),
+            ]
+
+        def total_seconds(td):
+            # timedelta.total_seconds() only added in 2.7
+            return td.seconds + td.days * 24 * 3600
+
+        seconds = [
+            total_seconds(t -
+                datetime(1904, 1, 1, 0, 0, 0, tzinfo=tdms.timezone))
+            for t in times]
+        fractions = [
+            int(float(t.microsecond) * 2 ** 58 / 5 ** 6)
+            for t in times]
+
         metadata = (
             # Number of objects
             "02 00 00 00"
@@ -478,45 +501,47 @@ class TDMSTestClass(unittest.TestCase):
             "00 00 00 00"
             # Number of properties (0)
             "00 00 00 00")
-        data = (
-            "00 00 00 00 00 00 00 60"
-            "01 00 00 00 00 00 00 00"
-            "00 00 00 00 00 00 00 60"
-            "02 00 00 00 00 00 00 00"
-            "00 00 00 00 00 00 00 60"
-            "03 00 00 00 00 00 00 00"
-            "00 00 00 00 00 00 00 60"
-            "04 00 00 00 00 00 00 00"
-        )
+        data = ""
+        for f, s in zip(fractions, seconds):
+            data += hexlify_value("<Q", f)
+            data += hexlify_value("<q", s)
+
+        def assertTimeEqual(a, b):
+            """Check times are equal, allowing for small difference
+            in microseconds due to floating point math being used
+            """
+
+            self.assertEqual(
+                a.replace(microsecond=0), b.replace(microsecond=0))
+            assert(abs(a.microsecond - b.microsecond) < 10)
 
         test_file = TestFile()
         toc = ("kTocMetaData", "kTocRawData", "kTocNewObjList")
         test_file.add_segment(metadata, data, toc)
-        tdms = test_file.load()
-        channel_data = tdms.channel_data("Group", "TimeChannel1")
+        tdmsData = test_file.load()
+        channel_data = tdmsData.channel_data("Group", "TimeChannel1")
         self.assertEqual(len(channel_data), 2)
-        self.assertEqual(channel_data[0][0], 1)
-        self.assertEqual(channel_data[1][0], 2)
+        assertTimeEqual(channel_data[0], times[0])
+        assertTimeEqual(channel_data[1], times[1])
         # Read fraction of second
-        self.assertEqual(channel_data[0][1], 0.375)
-        channel_data = tdms.channel_data("Group", "TimeChannel2")
+        channel_data = tdmsData.channel_data("Group", "TimeChannel2")
         self.assertEqual(len(channel_data), 2)
-        self.assertEqual(channel_data[0][0], 3)
-        self.assertEqual(channel_data[1][0], 4)
+        assertTimeEqual(channel_data[0], times[2])
+        assertTimeEqual(channel_data[1], times[3])
 
         # Now test it interleaved
         toc = toc + ("kTocInterleavedData", )
         test_file = TestFile()
         test_file.add_segment(metadata, data, toc)
-        tdms = test_file.load()
-        channel_data = tdms.channel_data("Group", "TimeChannel1")
+        tdmsData = test_file.load()
+        channel_data = tdmsData.channel_data("Group", "TimeChannel1")
         self.assertEqual(len(channel_data), 2)
-        self.assertEqual(channel_data[0][0], 1)
-        self.assertEqual(channel_data[1][0], 3)
-        channel_data = tdms.channel_data("Group", "TimeChannel2")
+        assertTimeEqual(channel_data[0], times[0])
+        assertTimeEqual(channel_data[1], times[2])
+        channel_data = tdmsData.channel_data("Group", "TimeChannel2")
         self.assertEqual(len(channel_data), 2)
-        self.assertEqual(channel_data[0][0], 2)
-        self.assertEqual(channel_data[1][0], 4)
+        assertTimeEqual(channel_data[0], times[1])
+        assertTimeEqual(channel_data[1], times[3])
 
     def test_time_track(self):
         """Add a time track to waveform data"""
@@ -524,9 +549,9 @@ class TDMSTestClass(unittest.TestCase):
         test_file = TestFile()
         (metadata, data, toc) = self.basic_segment()
         test_file.add_segment(metadata, data, toc)
-        tdms = test_file.load()
+        tdmsData = test_file.load()
 
-        obj = tdms.object("Group", "Channel2")
+        obj = tdmsData.object("Group", "Channel2")
         time = obj.time_track()
         self.assertEqual(len(time), len(obj.data))
         epsilon = 1.0E-15
@@ -579,14 +604,30 @@ class TDMSTestClass(unittest.TestCase):
             "00 00 00 00")
         data = ""
         test_file.add_segment(metadata, data, toc)
-        tdms = test_file.load()
-        data = tdms.channel_data("Group", "Channel1")
+        tdmsData = test_file.load()
+        data = tdmsData.channel_data("Group", "Channel1")
         self.assertEqual(len(data), 2)
         self.assertTrue(all(data == [1, 2]))
-        data = tdms.channel_data("Group", "Channel2")
+        data = tdmsData.channel_data("Group", "Channel2")
         self.assertEqual(len(data), 2)
         self.assertTrue(all(data == [3, 4]))
 
+    def test_memmapped_read(self):
+        """Test reading data into memmapped arrays"""
+
+        test_file = TestFile()
+        test_file.add_segment(*self.basic_segment())
+        tdmsData = test_file.load(memmap_dir=tempfile.gettempdir())
+
+        data = tdmsData.channel_data("Group", "Channel1")
+        self.assertEqual(len(data), 2)
+        self.assertEqual(data[0], 1)
+        self.assertEqual(data[1], 2)
+        data = tdmsData.channel_data("Group", "Channel2")
+        self.assertEqual(len(data), 2)
+        self.assertEqual(data[0], 3)
+        self.assertEqual(data[1], 4)
+
 
 if __name__ == '__main__':
     unittest.main()
